#### 多層パーセプトロン(MLP)による手書き文字認識
import os
import struct
import numpy as np
import matplotlib.pyplot as plt
from sklearn.decomposition import PCA
from sklearn.neural_network import MLPClassifier

# MNISTデータの読み込み関数
def load_mnist(path, kind='train'):

    labels_path = os.path.join(path,'%s-labels-idx1-ubyte'% kind)
    images_path = os.path.join(path,'%s-images-idx3-ubyte'% kind)
   
    with open(labels_path, 'rb') as lbpath:
        magic, n = struct.unpack('>II',lbpath.read(8))
        labels = np.fromfile(lbpath,dtype=np.uint8)
    
    with open(images_path, 'rb') as imgpath:
        magic, num, rows, cols = struct.unpack(">IIII",imgpath.read(16))
        images = np.fromfile(imgpath,dtype=np.uint8).reshape(len(labels), 784)
        
    return images, labels

# MNISTデータの読み込み
current_path = os.path.dirname(os.path.realpath("__file__"))
X_train, y_train = load_mnist(current_path, kind='train')
X_test, y_test = load_mnist(current_path, kind='t10k')

# 学習データとテストデータは最初のn_train_data, n_test_data個用いる
n_training_data = 5000
n_test_data = 5000

X_trn = X_train[:n_training_data][:]
y_trn = y_train[:n_training_data][:]
X_tst = X_test[:n_test_data][:]
y_tst = y_test[:n_test_data][:]

# PCAによる次元圧縮
n_components = 20
pca = PCA(n_components)
pca.fit(X_trn)
X_trn_pca = pca.transform(X_trn)
X_tst_pca = pca.transform(X_tst)

# MLPクラスのインスタンスを生成し，PCAで次元圧縮後の学習データに適合
nn = MLPClassifier(hidden_layer_sizes=(300, 200, 100), alpha=0.01, shuffle=False, random_state=1)
nn.fit(X_trn_pca, y_trn)

# 層の数を出力
print('number of layers=%d' % nn.n_layers_)

# Accuracyの算出
print('accuracy for training data: %.3f' % nn.score(X_trn_pca, y_trn))
print('accuracy for test data: %.3f' % nn.score(X_tst_pca, y_tst))

# 損失関数値のプロット
plt.figure(0)
plt.plot(range(len(nn.loss_curve_)), nn.loss_curve_)
plt.ylabel('Loss')
plt.xlabel('Epochs')
plt.tight_layout()

plt.show()

