#### SVMによるBreast Cancerデータの識別
#### 入れ子交差検証で最適パラメータを探索
import numpy as np
from sklearn import svm
from sklearn.datasets import load_breast_cancer
from sklearn.model_selection import StratifiedKFold, GridSearchCV
from sklearn.preprocessing import StandardScaler

# Breast Cancerデータのロード
df = load_breast_cancer()
X = df.data
y = df.target

# z標準化
sc = StandardScaler()
sc.fit(X)
X = sc.transform(X)

# 外側ループのための交差検証用データ生成インスタンス
kfold = StratifiedKFold(n_splits=10, shuffle=True)

acc_trn_list = []  #外側ループのfold毎の学習データに対するaccuracy格納用
acc_tst_list = []  #外側ループのfold毎のテストデータに対するaccuracy格納用

# グリッドサーチのパラメータリスト
# parametersに指定できる変数はscikit-learnのAPI referenceを参照のこと．
parameters = {'gamma':[0.01, 0.02, 0.05, 0.1, 0,2, 1.0], 'degree':[1,2,3]} 

# 内側ループでグリッドサーチを行う交差検証インスタンス
gs = GridSearchCV(svm.SVC(kernel='poly'), parameters, cv=2)

k=0
# 内側ループのグリッドサーチ
for train_itr, test_itr in kfold.split(X, y):
    gs.fit(X[train_itr], y[train_itr])
    print('Fold #{:2d}; Best Parameter:{}, Accuracy on validation data: {:.3f}'        .format(k+1,gs.best_params_,gs.best_score_))
    acc_trn_list.append(gs.score(X[train_itr],y[train_itr]))
    acc_tst_list.append(gs.score(X[test_itr],y[test_itr]))
    k=k+1

# 外側クロスバリデーションに対する平均正答率の表示
print('Average accuracy on training data: {:.3f}'.format(np.mean(acc_trn_list)))
print('Average accuracy on test data: {:.3f}'.format(np.mean(acc_tst_list)))

